﻿using MedicalDatabaseContracts;
using MedicalDatabaseContracts.Models;
using MedicalDatabaseContracts.SearchModels;
using MedicalDatabaseContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace MedicalView.Doctors
{
    public partial class FormDoctor : FormAbstractDetail
    {
        private readonly ILogger<FormDoctor> _logger;
        private readonly ILogic<Doctor, DoctorViewModel, DoctorSearchModel> _doctorLogic;
        private readonly ILogic<Specialization, SpecializationViewModel, SpecializationSearchModel> _specializationLogic;
        public FormDoctor(
            ILogger<FormDoctor> logger,
            ILogic<Doctor, DoctorViewModel, DoctorSearchModel> logic,
            ILogic<Specialization, SpecializationViewModel, SpecializationSearchModel> specializationLogic)
        {
            InitializeComponent();
            _logger = logger;
            _doctorLogic = logic;
            _specializationLogic = specializationLogic;
        }

        private void ApplyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxName.Text))
            {
                MessageBox.Show("Заполните имя", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            else if (string.IsNullOrEmpty(textBoxSurname.Text))
            {
                MessageBox.Show("Заполните фамилию", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            else if (string.IsNullOrEmpty(textBoxPhoneNumber.Text))
            {
                MessageBox.Show("Заполните номер телефона", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            else if (comboBoxSpecialization.SelectedIndex == -1 || comboBoxSpecialization.SelectedItem == null)
            {
                MessageBox.Show("Выберите специальность", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Сохранение модели врача");
            try
            {
                var model = new Doctor
                {
                    Id = ModelId ?? 0,
                    Name = textBoxName.Text,
                    Surname = textBoxSurname.Text,
                    Patronymic = textBoxPatronymic.Text,
                    PhoneNumber = textBoxPhoneNumber.Text,
                    SpecializationId = (comboBoxSpecialization.SelectedItem as SpecializationViewModel).Id
                };
                var operationResult = ModelId.HasValue ? _doctorLogic.Update(model) : _doctorLogic.Create(model);
                if (!operationResult)
                {
                    throw new Exception("Ошибка при сохранении. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка сохранения специальности");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void CancelToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Cancel();
        }

        private void FormDoctor_Load(object sender, EventArgs e)
        {
            if (ModelId.HasValue)
            {
                try
                {
                    _logger.LogInformation("Получение модели врача");
                    var view = _doctorLogic.ReadElement(ModelId.Value);
                    if (view != null)
                    {
                        textBoxName.Text = view.Name;
                        textBoxSurname.Text = view.Surname;
                        textBoxPatronymic.Text = view.Patronymic;
                        textBoxPhoneNumber.Text = view.PhoneNumber;

                        var specs = _specializationLogic.ReadList();
                        comboBoxSpecialization.DataSource = specs;
                        comboBoxSpecialization.SelectedIndex = specs.FindIndex(x => x.Id == view.SpecializationId);
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка получения модели врача");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
